#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
    2016/7/19  WeiYanfeng
    在 BrokerIB/bottle/CRepeatRunForWeb.py 基础上改进而来,
    通过 检测pid状态 控制某个命令服务运行。

    设计说明，请参见 ../readme.md

    WeiYF.20160721 从属于 CWebSupervisor.py 暂不整理实现。
"""

import sys
import os
import time
import json
import errno
import psutil  # subprocess

from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep, PrintInline, PrettyPrintStr


class CRepeatRunForWeb:
    """
        为WebServer进行运行封装
    """
    def __init__(self,sFNameConfig):
        self.sFNameConfig = sFNameConfig
        self._loadInitConfig(False)

        self.tmLastCheckRun = 0  # 上次检查运行时间，避免被调用太频繁
        self.iCheckCount = 0  # 检查计数

    def _loadInitConfig(self, bVerbose=True):
        self.dictConfig = {}
        with open(self.sFNameConfig,"r") as f:
            sData = f.read()
            # print "sData",sData
            # 这里configCmd.json要采用UTF-8编码，文件开头有UTF标记
            dictJson = json.loads(sData)
            for programId,dictParam in dictJson.items():
                if type(dictParam)==dict: # WeiYF.20150604是dict类型才执行
                    if not programId.startswith('<'):
                        self.dictConfig[programId] = LoadConfigByProgramId(self.sFNameConfig,programId,False)
        if bVerbose: PrintTimeMsg(PrettyPrintStr(self.dictConfig))
        self.dictCfgCmdOne = {}
        for programId,dictParam in self.dictConfig.items():
            rro = CRepeatRunOne(dictParam)
            self.dictCfgCmdOne[programId] = rro
        PrintTimeMsg("_loadInitConfig.Count=(%s)!" % (len(self.dictCfgCmdOne)))

    def ProgramCheckRun(self, bForce=False):
        secChkInterval = 30  # 检查时间最小间隔，也是被监控进程退出
        if bForce or int(time.time())-self.tmLastCheckRun>=secChkInterval:
            iRunCnt = 0
            for programId,rro in self.dictCfgCmdOne.items():
                if rro.CheckAndRunOneCmd(False)!=0:
                    iRunCnt += 1
            self.tmLastCheckRun = int(time.time())
            if iRunCnt>0 and False:
                PrintTimeMsg("ProgramCheckRun.RunCnt=%s=" % (iRunCnt))
            return True
        self.iCheckCount += 1
        bVerbose = False  # self.iCheckCount%10 == 0
        if bVerbose:
            PrintTimeMsg("ProgramCheckRun.ChkCnt=%s=" % (self.iCheckCount))
        return False

    def ProgramQueryAll(self, bVerbose=True):
        # 查询所有程序运行状态
        dictRet = {}
        for programId,rro in self.dictCfgCmdOne.items():
            dictRet[programId] = rro.cco.GetStatusDict()
        if bVerbose: PrintTimeMsg("ProgramQueryAll="+PrettyPrintStr(self.dictConfig))
        return dictRet

    def ProgramStopOne(self,sProgramId):
        # 停止某个程序
        rro = self.dictCfgCmdOne.get(sProgramId,None)
        if rro:
            rro.cco.CrtDelStopFile("CRT")
            bRet = rro.CheckAndRunOneCmd(False)
            PrintTimeMsg("ProgramStopOne(%s)=%s=" % (sProgramId,bRet))
            return bRet
        else:
            PrintTimeMsg("ProgramStopOne.get(%s)=Error" % (sProgramId))
            return False

    def ProgramStartOne(self,sProgramId):
        # 启动某个程序
        rro = self.dictCfgCmdOne.get(sProgramId,None)
        if rro:
            rro.cco.CrtDelStopFile("DEL")
            bRet = rro.CheckAndRunOneCmd(False)
            PrintTimeMsg("ProgramStartOne(%s)=%s=" % (sProgramId,bRet))
            return bRet
        else:
            PrintTimeMsg("ProgramStartOne.get(%s)=Error" % (sProgramId))
            return False

    def ProgramRestartOne(self,sProgramId):
        # 启动某个程序
        self.ProgramStopOne(sProgramId)
        self.ProgramStartOne(sProgramId)

    def ProgramGetLogTailTxt(self,sProgramId):
        rro = self.dictCfgCmdOne.get(sProgramId,None)
        if rro:
            return rro.cco.GetLogTailTxt()
        else:
            return "ProgramGetLogTailTxt(%s)=Error" % (sProgramId)


def LoopCRepeatRunMany(sFNameConfig="configCmd.json"):
    # WeiYF.20150610 该功能实现后，事实上就可以将
    rrfw = CRepeatRunForWeb(sFNameConfig)
    iCnt = 0
    while True:
        bRet = rrfw.ProgramCheckRun()
        # rrfw.ProgramRestartOne("TestHandleTradeFmRedis")
        # rrfw.ProgramQueryAll()
        #
        iCnt += 1
        bVerbose = iCnt%60 == 0
        PrintAndSleep(10,"*** LoopCRepeatRunMany.ProgramCheckRun(iCnt=%s)" % (iCnt),bVerbose)


def stdinCRepeatRunOne():
    # c.LoopWaitCmdExec('ping juchecar.com -t')
    # c.LoopWaitCmdExec('ping juchecar.com -t')
    # c.LoopWaitCmdExec(['ping', 'juchecar.com','-t'])
    # c.LoopWaitCmdExec('ping www.juchecar.com')
    # c.LoopWaitCmdExec(r'C:\Python27\python.exe TcpChannelWin.py')
    #
    # dictP = {  'cmdExec': u'C:\\Python27\\python.exe .\\funcIB\\CTwsTransferMsgToRedis.py',
    #    'cmdTitle': u'\u8f6c\u9001\u884c\u60c5\u4fe1\u606f\u5230Redis',
    #    'endFlag': u'JustForEnd',
    #    'logProgramId': u'D:\\WeiYFGitSrc\\PythonProject\\BrokerIB\\log\\CTwsTransferMsgToRedis',
    #    'workDir': u'D:\\WeiYFGitSrc\\PythonProject\\BrokerIB\\'
    # }
    dictP = {  'cmdExec': u'ping juchecar.com',  #  -t #dir C:\\
       'cmdTitle': u'\u8f6c\u9001\u884c\u60c5\u4fe1\u606f\u5230Redis',
       'endFlag': u'JustForEnd',
       'err2out': u'True',
       'ProgramId': u'CTwsTransferMsgToRedis',
       'logDir': u'D:\\WeiYFGitSrc\\PythonProject\\BrokerIB\\log\\',
       'workDir': u'D:\\WeiYFGitSrc\\PythonProject\\BrokerIB\\'
    }
    sJsonParam = sys.stdin.read()
    print "sJsonParam=",sJsonParam
    dictP = json.loads(sJsonParam)

    print "dictP=",PrettyPrintStr(dictP)
    StartCRepeatRunOne(dictP)
    return


def testPrintStdIn():
    """
        WeiYF.20150603
        虽然Popen可以通过 communicate 传递JSON数据串，但会因此阻塞而无法继续执行;
        虽然 StringIO 可以模拟将串转为 file object ，但这种对象无法在POpen环境下使用。
        http://stackoverflow.com/questions/20568107/python-stringio-for-popen
        http://stackoverflow.com/questions/163542/python-how-do-i-pass-a-string-into-subprocess-popen-using-the-stdin-argument
    """
    sJsonParam = sys.stdin.read()
    print "sJsonParam=",sJsonParam
    sys.exit(-1)

# ----------------------------------
if __name__ == '__main__':
    # testCConfigCmdOne()
    # testPrintStdIn()
    # stdinCRepeatRunOne()
    # mainCRepeatRunOne() #WeiYF.20150610 用于 runRepeatConfigBat.py程序调用，这种模式淘汰
    LoopCRepeatRunMany('configWeb.json')




#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
2016/7/21  WeiYanfeng

weberWinSV 包实现了Windows 平台下的程序管理工具，类似于Linux系统下的 Supervisor 。
该包实际内含两个管理工具：
1. CWinSupervisor 是启动管理其它“命令行程序”的管理工具。可以单独部署。
2. CWebSupervisor 是监控 CWinSupervisor 运行状态的工具。可以根据需要部署。


# pip install psutil
# pip install import_file
"""
from .CWinSupervisor import StartCWinSupervisor
from .CCheckStdoutRestart import CCheckStdoutRestart

if __name__ == '__main__':
    pass



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
2016/7/21  WeiYanfeng

依据参数配置，分组启动并控制其它 cmdProgram=命令行程序 运行的工具。
参数配置说明请参见 demoSettingsWinSV.py 单元。

这些 cmdProgram 具有如下特征：
1. 一般作为服务程序运行。
2. 没有图形界面。
3. 有标准输出和标准错误输出，但没有标准输入。

CWinSupervisor 的控制运行单个 cmdProgram 的逻辑如下：
1. 若 stopFile 存在，则直接控制相应程序退出。
2. 若 pidFile 存在，则检查其对应的pid是否存在，若存在，则继续检测该服务。
3. 进入到 workDir，启动相应命令，并
    A.将其输出重定向到 outFile 和 errFile 中。
    B.将其pid记录到 pidFile 中。
    C.将启动服务日志追加方式记录到 workDir下的 logWeberWinSV.txt 中
4.循环1分钟：
    A.检测对应进程状态，并更新 pidFile 的时间，以表明对应命令正常运行。
    B.检测 stopFile 是否存在，若存在，则终止对应进程运行。
"""

import sys
import os

from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep
from .winsvFuncs import LoadWinSVConfigFmFile
from .CWinSVProgram import CWinSVProgram


class CWinSupervisor:
    def __init__(self, srcfile, sPythonFileName, sDictVarName, sLongIdStr):
        # PrintTimeMsg("CWinSupervisor.__init__.srcfile=%s=" % srcfile)
        # PrintTimeMsg("CWinSupervisor.__init__.sPythonFileName=%s=" % sPythonFileName)
        # PrintTimeMsg("CWinSupervisor.__init__.sDictVarName=%s=" % sDictVarName)
        # PrintTimeMsg("CWinSupervisor.__init__.sLongIdStr=%s=" % sLongIdStr)
        self.sStartCWD = os.getcwd()
        # PrintTimeMsg("CWinSupervisor.__init__.sStartCWD=%s=" % self.sStartCWD)
        self.lsProgram = LoadWinSVConfigFmFile(srcfile, sPythonFileName, sDictVarName, sLongIdStr)
        iProgramCnt = len(self.lsProgram)
        if iProgramCnt == 0:
            PrintTimeMsg("CWinSupervisor.__init__.iProgramCnt=(%d)Error EXIT!" % iProgramCnt)
            sys.exit(-1)
        PrintTimeMsg("CWinSupervisor.__init__.iProgramCnt=(%d)!" % iProgramCnt)

        self.iCheckIntervalSeconds = 60

        iCmdCnt = 0
        self.lsWinSV = []
        for dictParam in self.lsProgram:
            oWinSV = CWinSVProgram(dictParam)
            self.lsWinSV.append(oWinSV)
            PrintTimeMsg("  lsWinSV[%.2d]=(%s)!" % (iCmdCnt, oWinSV.sCmdExec))
            iCmdCnt += 1

    def __del__(self):
        pass

    def LoopAndWatchPrograms(self):
        iLoopCnt = 0
        while True:
            os.chdir(self.sStartCWD)  # 回到启动时的目录
            iStart, iStop = 0, 0
            for oWinSV in self.lsWinSV:
                iChg = oWinSV.CheckAndRunOneCmd(iLoopCnt)
                if iChg > 0:
                    iStart += 1
                if iChg < 0:
                    iStop += 1
            if iStart > 0 or iStop > 0:
                PrintTimeMsg("  >>(%s)#%d.iStart=%s,iStop=%s!" % (
                    self.sStartCWD, iLoopCnt, iStart, iStop))
            else:
                PrintAndSleep(self.iCheckIntervalSeconds,
                              "  >>(%s)#%d" % (self.sStartCWD, iLoopCnt),
                              iLoopCnt % 10 == 0)
            iLoopCnt += 1


def StartCWinSupervisor(srcfile, sPythonFileName, sDictVarName, sLongIdStr):
    ws = CWinSupervisor(srcfile, sPythonFileName, sDictVarName, sLongIdStr)
    ws.LoopAndWatchPrograms()

# --------------------------------------
if __name__ == '__main__':
    sPythonFileName = 'demoSettingsWinSV.py'
    sDictVarName = 'gDictConfigByGroupId'
    sLongIdStr = 'groupExample.programDirC,groupExample.programQQt'
    # sLongIdStr = 'groupExample.programQQ'
    # sLongIdStr = 'groupExample.programQQt'
    if len(sys.argv) >= 4:
        sPythonFileName = sys.argv[1]
        sDictVarName = sys.argv[2]
        sLongIdStr = sys.argv[3]
    StartCWinSupervisor(__file__, sPythonFileName, sDictVarName, sLongIdStr)



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
2016/7/20  WeiYanfeng
针对单个 cmdProgram=命令行程序 相关操作进行的封装。

-------- 依赖包
pip install psutil

"""

import sys
import os
import psutil  # subprocess

from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep, PrintInline, PrettyPrintStr
from .winsvFuncs import ReadTailLines, TerminateByPid


class CWinSVProgram:
    def __init__(self, dictParam):
        self.dictParam = dictParam
        PrintTimeMsg("CWinSVProgram.dictParam=%s" % PrettyPrintStr(self.dictParam))
        self.sGroupId = self.dictParam.get('groupId','')
        self.sProgramId = self.dictParam.get('programId','')
        self.sCmdExec = self.dictParam.get('cmdExec','')
        self.sWorkDir = self.dictParam.get('workDir','')
        self.sLogDir = self.dictParam.get('logDir','')
        self.bErr2Out = self.dictParam.get('err2Out',True)
        self.bShellPopen = self.dictParam.get('shellPopen',False)

        # PrintTimeMsg("CWinSVProgram.CWinSVProgram(%s.%s)=%s=" % (
        #     self.sGroupId, self.sProgramId, self.sCmdExec))
        # PrintTimeMsg("CWinSVProgram.sWorkDir=%s=" % (self.sWorkDir))
        # PrintTimeMsg("CWinSVProgram.sLogDir=%s=,bErr2Out=%s" % (self.sLogDir, self.bErr2Out))
        #   stopFile=<logDir>/<programId>.stop stop文件，若存在，则终止对应进程
        #   pidFile=<logDir>/<programId>.pid pid文件，用于查询该进程运行状态
        #   outFile=<logDir>/<programId>.out 标准输出日志文件
        #   errFile=<logDir>/<programId>.err 标准错误输出日志文件
        #   outerrFile=<logDir>/<programId>.tr 标准输出和错误输出合并日志文件

        sLogProgramId = self.sLogDir+self.sProgramId
        if self.bErr2Out:
            self.sLogOutFile = sLogProgramId+".tr"
            self.sLogErrFile = self.sLogOutFile
        else:
            self.sLogOutFile = sLogProgramId+".out"
            self.sLogErrFile = sLogProgramId+".err"
        self.sPidFile = sLogProgramId+".pid"
        self.sStopFile = sLogProgramId+".stop"

        try:
            sDir = self.sLogDir
            os.makedirs(sDir)
        except OSError as exc:  # Python >2.5 (except OSError, exc: for Python <2.5)
            import errno
            if exc.errno == errno.EEXIST and os.path.isdir(sDir):
                pass
            else:
                raise

        self.fdOut = None
        self.fdErr = None
        self.Process = None  # 当前在运行的进程对象

        self.bVerbose = False
        # self.bVerbose = True #WeiYF.20160818 减少日志

    def __del__(self):
        self.__closeFd()

    def CheckStopFileExists(self):
        # 检查 stop文件 是否存在
        sFN = self.sStopFile
        bRet = os.path.exists(sFN)
        if bRet:
            PrintTimeMsg("CheckStopFileExists(%s)=%s" % (sFN,bRet))
        return bRet

    def CrtDelStopFile(self, sAction):
        # 创建或删除 stop文件
        try:
            sFN = self.sStopFile
            if sAction == "DEL":
                if os.path.exists(sFN):
                    os.remove(sFN)
                    PrintTimeMsg("CrtDelStopFile.%s(%s)OK" % (sAction,sFN))
                else:
                    PrintTimeMsg("CrtDelStopFile.%s(%s)NotExists" % (sAction,sFN))
            else:
                if os.path.exists(sFN):
                    PrintTimeMsg("CrtDelStopFile.%s(%s)NotExists" % (sAction,sFN))
                else:
                    with open(sFN,"a") as f:
                        f.write('%s' % (GetCurrentTime()))  # 默认写入当前时间
                    PrintTimeMsg("CrtDelStopFile.%s(%s)OK" % (sAction,sFN))
        except OSError as e:
            PrintTimeMsg("CrtDelStopFile(%s)Error=(%s)!" % (self.sStopFile,str(e)))

    def CheckPidFileExists(self):
        # 检查 pid文件 是否存在
        bRet = os.path.exists(self.sPidFile)
        if not bRet:
            PrintTimeMsg("CheckPidFileExists(%s)=%s" % (self.sPidFile,bRet))
        return bRet

    def WritePidToFile(self,pid):
        # 输出 pid 文件
        try:
            with open(self.sPidFile,"w") as f:  # 覆盖模式输出
                sS = "%s" % (pid)
                f.write(sS)
            # PrintTimeMsg("_WritePidToFile(%s,%s)OK!" % (self.sPidFile,pid))
        except OSError as e:
            PrintTimeMsg("WritePidToFile(%s)Error=(%s)!" % (self.sPidFile,str(e)))

    def ReadPidFromFile(self):
        # 从文件中读取 pid
        try:
            with open(self.sPidFile,"r") as f:  # 覆盖模式输出
                sS = f.read()
                sS = sS.strip('\n')   # 删除结尾换行
                pid = int(sS)
                # PrintTimeMsg("ReadPidFromFile(%s,%s)OK!" % (self.sPidFile,pid))
                return pid
        except OSError as e:
            PrintTimeMsg("ReadPidFromFile(%s)Error=(%s)!" % (self.sPidFile,str(e)))
            return -1

    def TouchPidFile(self):
        # 更新 pid文件 时间
        try:
            os.utime(self.sPidFile, None) #
            # PrintTimeMsg("TouchPidFile(%s)OK!" % (self.sPidFile))
        except OSError as e:
            PrintTimeMsg("TouchPidFile(%s)Error=(%s)!" % (self.sPidFile,str(e)))

    def GetPidFileTimeInt(self):
        # 读取 pid 文件修改时间
        try:
            # statinfo = os.stat('C:\\')
            # statinfo = os.stat('..\\test\\testXINA50.py')
            statinfo = os.stat(self.sPidFile)
            mtime = int(statinfo.st_mtime)
            # PrintTimeMsg("GetPidFileTimeInt(%s)=%s!" % (self.sPidFile,mtime))
            return mtime
        except OSError as e:
            PrintTimeMsg("GetPidFileTimeInt(%s)Error=(%s)!" % (self.sPidFile,str(e)))
            return -1

    def GetRedirectCmd(self):
        # WeiYF.20150603 这种方式需要 Shell=True 放弃
        sCmd = "%s 1> %s 2>&1" % (self.sCmdExec, self.sLogOutFile)
        PrintTimeMsg("GetRedirectCmd=(%s)" % sCmd)
        return sCmd

    def ChangeWorkDir(self):
        # 更改工作目录
        # sOldDir = os.getcwd()
        os.chdir(self.sWorkDir)
        # sNewTmp = os.getcwd()
        # PrintTimeMsg("ChangeWorkDir(%s->%s)" % (sOldDir,sNewTmp))

    def GetLogFileInfo(self):
        # 返回日志文件信息
        try:
            # statinfo = os.stat('C:\\')
            # statinfo = os.stat('..\\test\\testXINA50.py')
            statinfo = os.stat(self.sLogOutFile)
            # PrintTimeMsg("GetLogFileInfo(%s)=%s!" % (self.sLogOutFile,statinfo))
            return statinfo
        except OSError as e:
            PrintTimeMsg("GetLogFileInfo(%s)Error=(%s)!" % (self.sLogOutFile,str(e)))
            return -1

    def __closeFd(self):
        if self.fdOut:
            self.fdOut.close()
            self.fdOut = None
        if self.fdErr:
            self.fdErr.close()
            self.fdErr = None

    def CheckAndRunOneCmd(self, iLoopCnt):
        # 检查并启动命令，适合于嵌入到主进程内运行
        #   返回+1，表示 启动了新程序
        #   返回-1，表示 停止了新程序
        #   返回0，表示 没变化
        self.ChangeWorkDir()
        # sCmd = self.getRedirectCmd()
        sCmd = self.sCmdExec
        if self.CheckStopFileExists():
            # Need Quit
            iRet = 0
            if self.CheckPidFileExists():
                pid = self.ReadPidFromFile()
                if psutil.pid_exists(pid):
                    TerminateByPid(pid)
                    iRet = -1
                else:
                    if self.bVerbose:
                        PrintTimeMsg("Check(%s)AlreadyQuit.pid=(%s)..." % (sCmd, pid))
            else:
                PrintTimeMsg("checkPidFileExists.FALSE=(%s)..." % sCmd)
            return iRet
        else:
            if iLoopCnt > 0 and self.CheckPidFileExists():
                # WeiYF.20170928 非首次启动才检查pid文件
                pid = self.ReadPidFromFile()
                if psutil.pid_exists(pid):
                    self.TouchPidFile()
                    if self.bVerbose:
                        PrintTimeMsg("Check(%s)StillRunning.pid=(%s)..." % (sCmd, pid))
                    return 0
            self.__closeFd()
            if self.bVerbose:
                PrintTimeMsg("CheckAndRunOneCmd.sLogOutFile=(%s)..." % self.sLogOutFile)
            self.fdOut = open(self.sLogOutFile, 'a+')
            self.fdOut.write("\n[%s]%s{%s}...\n" % (GetCurrentTime(), '*'*40, self.sProgramId))
            self.fdOut.flush()
            if self.bErr2Out:
                self.fdErr = self.fdOut
            else:
                self.fdErr = open(self.sLogErrFile, 'a+')
            # self.Process = psutil.Popen(sCmd, stdout=self.fdOut,stderr=self.fdErr, shell=False) #True
            self.Process = psutil.Popen(sCmd, stdout=self.fdOut,
                                        stderr=self.fdErr, shell=self.bShellPopen)
            # Shell=True会新起一个cmd进程
            pid = self.Process.pid
            # sMsg = "%s#Start(%s)=%s" % (iStartCnt,sCmd,pid)
            # self.recordPopenLog(sMsg)
            PrintTimeMsg("Popen(%s)OK.pid=%s" % (sCmd,pid))  # [20:]
            self.WritePidToFile(pid)
            return +1

    # 如下函数用于 CWebSupervisor
    def GetLogTailTxt(self, nLine=10):
        # 读取日志文件结尾若干行
        return ReadTailLines(self.sLogOutFile,nLine)

    def GetStatusDict(self):
        # 返回当前命令运行状态字典
        dictRet = self.dictParam
        pid = self.ReadPidFromFile()
        dictRet['pid'] = pid
        dictRet['pidExists'] = 'T' if psutil.pid_exists(pid) else 'F'
        dictRet['tmPidFile'] = self.GetPidFileTimeInt()
        statinfo = self.GetLogFileInfo()
        dictRet['tmLogFile'] = statinfo.st_mtime
        dictRet['szLogFile'] = statinfo.st_size
        # dictRet['logTail5'] = ReadTailLines(self.sLogOutFile,5)
        return dictRet


def testCWinSVProgram():
    dictParam = { 'cmdExec': u'ping juchecar.com',  # -t # dir C:\\
        'cmdTitle': u'\u8f6c\u9001\u884c\u60c5\u4fe1\u606f\u5230Redis',
        'endFlag': u'JustForEnd',
        'err2out': u'True',
        'groupId': u'groupId',
        'programId': u'ProgramId',
        'logDir': u'D:\\WeiYFGitSrc\\PythonProject\\WeberEgg\\GitHub_PythonEggs\\weberWinSV\\weberWinSV\\test\\log\\',
        'workDir': u'D:\\WeiYFGitSrc\\PythonProject\\WeberEgg\\GitHub_PythonEggs\\weberWinSV\\weberWinSV\\test\\'
    }
    c = CWinSVProgram(dictParam)
    c.CheckStopFileExists()
    c.CheckPidFileExists()
    c.GetRedirectCmd()
    c.GetPidFileTimeInt()
    c.GetLogFileInfo()
    print(ReadTailLines(c.sLogOutFile,3))  # 10

    c.CrtDelStopFile("CRT")
    PrintAndSleep(10, "*** CrtDelStopFile")
    c.CrtDelStopFile("DEL")

    print(c.CheckAndRunOneCmd())

# ------------------------------------------------
if __name__ == '__main__':
    testCWinSVProgram()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
    2016/7/21  WeiYanfeng
    在Windows下启动并控制其它命令行程序(cmdProgram)运行
    类似于Linux系统下的 Supervisor 工具

    简化实现方式：
      1.WebServer启动时，启动 runRepeatConfigBat.py 并记录其进程号；
      2.WebServer提供对该进程提供 停止、重启服务；
      3.WebServer提供核心进程状态查询服务；

    实现原理描述：
      1.从当前目录下 configCmd.json 中读取要管理的程序；
      2.检查对应的pid文件是否存在，若pid文件存在且对应pid也存在，则跳过。
      3.否则采用NoWait方式启动对应程序，并记录其pid到pid文件。
      4.同时该启动程序会将程序输出重定向到日志文件，并在对应程序退出情况下，将其自动重启。
      5.提供URL，能够：A)查询到服务程序状态；B)暂停、启动、重启指定服务；
"""
import _addpath
import sys

import os
import time

import json
from cpyf.publicFuncs import PrintTimeMsg,GetCurrentTime,PrettyPrintStr,GetUnixTimeLocal

# import psutil

from CRepeatRunForWeb import CRepeatRunForWeb

from local_bottle import route, template, response, request
from local_bottle import static_file, auth_basic, parse_auth

gCmdUrlPrefix = "/cmd"   # URL前缀

gRRFW = CRepeatRunForWeb("configCmd.json")  # 缺省配置文件
gRRFW.ProgramCheckRun()


def html_ListProgramAll(numTx=100):
    global gRRFW
    # return "<pre>%s</pre>" % PrettyPrintStr(gRRFW.ProgramQueryAll())

    sRows = u'<tr><th>状态描述</th><th>备注</th><th>程序名</th><th>操作</th></tr>\n'
    dictAll = gRRFW.ProgramQueryAll(False)
    for programId,dictStatus in dictAll.items():
        cmdExec = dictStatus.get("cmdExec","")
        cmdTitle = dictStatus.get("cmdTitle","")
        # logTail5 = dictStatus.get("logTail5","")
        pid = dictStatus.get("pid",0)
        bPidExists = dictStatus.get("pidExists",'F')=='T'
        szLogFile = dictStatus.get("szLogFile",0)
        tmPidFile = dictStatus.get("tmPidFile",0)
        sStatus = ''
        if time.time()-tmPidFile>60:
            if bPidExists:
                sStatus = 'Wrong'
            else:
                sStatus = "Suspend"
        else:
            if bPidExists:
                sStatus = 'Running'
            else:
                sStatus = "JustQuit"
        sTimePid = GetUnixTimeLocal(int(tmPidFile))
        sD = sTimePid[0:8]
        sT = sTimePid[-6:]
        sTm = '%s-%s-%s %s:%s:%s' % (sD[0:4],sD[4:6],sD[6:8],sT[0:2],sT[2:4],sT[4:6])
        htmlDesc = '%s@%s [%s]' % (sStatus,sTm,pid)
        htmlMemo = cmdTitle
        htmlName = '<a href="%s/" title="%s">%s</a>' % (gCmdUrlPrefix, cmdExec, programId)
        htmlRestart = '<a href="%s/ProgramRestart/%s">Restart</a>' % (gCmdUrlPrefix, programId)
        htmlStop = '<a href="%s/ProgramStop/%s">Stop</a>' % (gCmdUrlPrefix, programId)
        htmlStart = '<a href="%s/ProgramStart/%s">Start</a>' % (gCmdUrlPrefix, programId)
        # htmlStartStop = htmlStop if bPidExists else htmlStart
        if bPidExists:
            htmlStart = ''
        htmlLogTailTxt = '<a href="%s/GetLogTailTxt/%s">TailTxt</a>' % (gCmdUrlPrefix, programId)
        htmlAction = '%s %s %s %s' % (htmlRestart, htmlStart, htmlStop, htmlLogTailTxt)

        # a_AddressTxid = '<a href="https://blockchain.info/tx/%s" target ="_blank">%s</a>' % (txId,sAddress)
        sRow = '<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\n' % (
                htmlDesc, htmlMemo, htmlName, htmlAction)
        sRows += sRow
    return '<div align="center"><table border="0" cellspacing="1" class="table-striped">\n%s</table></div>\n' % (sRows)


def checkUserPasswd(sUser, sPasswd):
    sClientIP = request.environ.get('REMOTE_ADDR','')
    sAddition = "%s,%s" % (request.url,request.remote_route)
    sMsg = "user=%s,passwd=%s,memo=%s" % (sUser,sPasswd,sAddition)
    sTag = "Deny"
    bRet = (sUser=="btcTws" and sPasswd=="796796")
    if bRet:
        sTag = "Pass"
        sMsg = "memo=%s" % (sAddition)
    with open("ip_check.txt","a") as f: #追加模式输出
        sS = "[%s]%s:%s\t#%s\n" % (GetCurrentTime(),sTag, sClientIP, sMsg)
        f.write(sS)
    return bRet


@route(gCmdUrlPrefix)
@route(gCmdUrlPrefix+'/')
@auth_basic(checkUserPasswd)
def cmdRoot():
    # print gRRFW.ProgramCheckRun()
    return html_ListProgramAll()


@route(gCmdUrlPrefix+'/ProgramCheckRun')
def cmdProgramCheckRun():
    global gRRFW
    bRet = gRRFW.ProgramCheckRun()
    sRet = "Ignore"
    if bRet:
        sRet = "Check"
    return sRet


def getHtmlHome():
    global gCmdUrlPrefix
    return '<a href="%s">ReturnBack</a>' % (gCmdUrlPrefix)


@route(gCmdUrlPrefix+'/GetLogTailTxt/<sProgramId>')
@auth_basic(checkUserPasswd)
def cmdGetLogTailTxt(sProgramId):
    global gRRFW
    # print gRRFW.ProgramCheckRun()
    lsLines = gRRFW.ProgramGetLogTailTxt(sProgramId)
    sText = ''
    for sLine in lsLines:
        sText += sLine
    return '<pre>%s</pre>%s' % (sText,getHtmlHome())


@route(gCmdUrlPrefix+'/ProgramStop/<sProgramId>')
@auth_basic(checkUserPasswd)
def cmdProgramStop(sProgramId):
    global gRRFW
    bRet = gRRFW.ProgramStopOne(sProgramId)
    gRRFW.ProgramCheckRun(True)
    return 'ProgramStopOne(%s)=(%s)<br/>%s' % (sProgramId,bRet,getHtmlHome())


@route(gCmdUrlPrefix+'/ProgramStart/<sProgramId>')
@auth_basic(checkUserPasswd)
def cmdProgramStart(sProgramId):
    global gRRFW
    bRet = gRRFW.ProgramStartOne(sProgramId)
    gRRFW.ProgramCheckRun(True)
    return 'ProgramStartOne(%s)=(%s)<br/>%s' % (sProgramId,bRet,getHtmlHome())


@route(gCmdUrlPrefix+'/ProgramRestart/<sProgramId>')
@auth_basic(checkUserPasswd)
def cmdProgramRetart(sProgramId):
    global gRRFW
    bRet = gRRFW.ProgramStartOne(sProgramId)
    gRRFW.ProgramCheckRun(True)
    return 'ProgramStartOne(%s)=(%s)<br/>%s' % (sProgramId,bRet,getHtmlHome())

if __name__ == '__main__':
    print html_ListProgramAll()
